//============================================================================
// Name        : acat.cpp
// Author      : Martin Zak
//============================================================================

#include <iostream>
#include <string>
#include <set>
#include <cstdlib>
#include <unistd.h>
#include <exception>
#include <exception/AlibException.h>
#include <factory/DataFactory.hpp>
#include <sax/SaxParseInterface.h>
#include <sax/ParserException.h>
#include <object/Object.h>
#include <std/list.hpp>


#define VERSION "0.0.1"
#define USAGE "Usage: catPDA [-c] [input]";

int main(int argc, char** argv) {
	if(argc == 2 && std::string(argv[1])=="-v" ) {
			std::cout << argv[0] << " version " << VERSION << "\n";
			std::cout << USAGE;
			std::cout << std::endl;
			return EXIT_SUCCESS;
		}

	int i=getopt(argc, argv, "t:");
	while(i!=-1) {
		switch(i){
		default:
			optind--;
			break;
		}
		i=getopt(argc, argv, "t:");
	}

	try {
		std::list<sax::Token> tokens;
		if(optind == argc) {
			std::string input(std::istreambuf_iterator<char>(std::cin), std::istreambuf_iterator<char>());
			sax::SaxParseInterface::parseMemory(input, tokens);
		} else {
			sax::SaxParseInterface::parseFile(argv[optind],tokens);
		}

		alib::Object object = alib::DataFactory::fromTokens<alib::Object>(tokens);
		alib::DataFactory::toStdout(object);

		return 0;
	} catch(const exception::AlibException& exception) {
		alib::DataFactory::toStdout(exception);
		return 1;
	} catch(...) {
		return 127;
	}
}
