/*
 * GasTexConverter.h
 *
 *  Created on: Jan 1, 2014
 *      Author: Martin Zak
 */

#ifndef GAS_TEX_CONVERTER_H_
#define GAS_TEX_CONVERTER_H_

#include <ostream>
#include <map>
#include <utility>
#include <vector>
#include "automaton/Automaton.h"
#include "alphabet/Symbol.h"

class GasTexConverter : public automaton::VisitableAutomatonBase::const_visitor_type {
	void Visit(void*, const automaton::UnknownAutomaton& automaton) const;
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;

	static void printTransitionMap( const std::map<std::pair<std::string, std::string>, std::string> transitionMap, std::ostream& out);
	static std::string getStackSymbols(const std::vector<alphabet::Symbol>& stackSymbols);

	static void transitions(const automaton::UnknownAutomaton& fsm, std::ostream& out);
	static void transitions(const automaton::EpsilonNFA& fsm, std::ostream& out);
	static void transitions(const automaton::NFA& fsm, std::ostream& out);
	static void transitions(const automaton::DFA& fsm, std::ostream& out);
	static void transitions(const automaton::ExtendedNFA& fsm, std::ostream& out);
	static void transitions(const automaton::CompactNFA& fsm, std::ostream& out);
	static void transitions(const automaton::DPDA& pda, std::ostream& out);
	static void transitions(const automaton::SinglePopDPDA& tm, std::ostream& out);
	static void transitions(const automaton::InputDrivenNPDA& pda, std::ostream& out);
	static void transitions(const automaton::VisiblyPushdownDPDA& tm, std::ostream& out);
	static void transitions(const automaton::VisiblyPushdownNPDA& tm, std::ostream& out);
	static void transitions(const automaton::RealTimeHeightDeterministicDPDA& tm, std::ostream& out);
	static void transitions(const automaton::RealTimeHeightDeterministicNPDA& tm, std::ostream& out);
	static void transitions(const automaton::NPDA& pda, std::ostream& out);
	static void transitions(const automaton::SinglePopNPDA& tm, std::ostream& out);
	static void transitions(const automaton::OneTapeDTM& tm, std::ostream& out);

	static const GasTexConverter instance;
public:
	static void convert(const automaton::Automaton& a, std::ostream& out);

	static void convert(const automaton::UnknownAutomaton& a, std::ostream& out);
	static void convert(const automaton::EpsilonNFA& a, std::ostream& out);
	static void convert(const automaton::NFA& a, std::ostream& out);
	static void convert(const automaton::DFA& a, std::ostream& out);
	static void convert(const automaton::ExtendedNFA& a, std::ostream& out);
	static void convert(const automaton::CompactNFA& a, std::ostream& out);
	static void convert(const automaton::DPDA& a, std::ostream& out);
	static void convert(const automaton::SinglePopDPDA& a, std::ostream& out);
	static void convert(const automaton::InputDrivenNPDA& a, std::ostream& out);
	static void convert(const automaton::VisiblyPushdownDPDA& a, std::ostream& out);
	static void convert(const automaton::VisiblyPushdownNPDA& a, std::ostream& out);
	static void convert(const automaton::RealTimeHeightDeterministicDPDA& a, std::ostream& out);
	static void convert(const automaton::RealTimeHeightDeterministicNPDA& a, std::ostream& out);
	static void convert(const automaton::NPDA& a, std::ostream& out);
	static void convert(const automaton::SinglePopNPDA& a, std::ostream& out);
	static void convert(const automaton::OneTapeDTM& a, std::ostream& out);
};

#endif /* GAS_TEX_CONVERTER_H_ */
