/*
 * FSMTotal.h
 *
 *  Created on: 20. 9. 2014
 *	  Author: Tomas Pecka
 */

#ifndef FSM_TOTAL_H_
#define FSM_TOTAL_H_

#include <algorithm>
#include <deque>
#include <set>

#include "automaton/Automaton.h"
#include <automaton/common/State.h>

namespace automaton {

/**
 * Makes finite automaton's transition function total.
 * Source: Melichar: Algorithm 2.22
 */
class FSMTotal : public VisitableAutomatonBase::const_visitor_type {
public:
	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static automaton::Automaton total(const automaton::Automaton& automaton);

	static automaton::NFA total(const automaton::NFA& automaton);
	static automaton::DFA total(const automaton::DFA& automaton);

private:
	void Visit(void*, const UnknownAutomaton& automaton) const;
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;

	static const FSMTotal FSM_TOTAL;
};

}

#endif /* FSM_TOTAL_H_ */
