/*
 * RHPDAToPDA.h
 *
 *  Created on: 23. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef RHPDA_TO_PDA_H_
#define RHPDA_TO_PDA_H_

#include <algorithm>
#include <deque>
#include <set>

#include "automaton/Automaton.h"
#include <automaton/common/State.h>

namespace automaton {

class RHPDAToPDA : public VisitableAutomatonBase::const_visitor_type {
public:
	static automaton::DPDA convert( const automaton::RealTimeHeightDeterministicDPDA & pda);
	static automaton::DPDA convert( const automaton::DPDA & pda);
	static automaton::NPDA convert( const automaton::RealTimeHeightDeterministicNPDA & pda);
	static automaton::NPDA convert( const automaton::NPDA & pda);

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static automaton::Automaton convert( const automaton::Automaton & automaton );
private:
	void Visit(void*, const UnknownAutomaton& automaton) const;
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;

	static const RHPDAToPDA RHPDA_TO_PDA;
};

}

#endif /* RHPDA_TO_PDA_H_ */
