/*
 * Algebraic.h
 *
 *  Created on: 11. 2. 2014
 *	  Author: Tomas Pecka
 */

#ifndef FA2RE_ALGEBRAIC_H_
#define FA2RE_ALGEBRAIC_H_

#include <deque>
#include <map>
#include <queue>

#include <regexp/RegExp.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>
#include <automaton/Automaton.h>

namespace conversions
{

namespace fa2re
{

// http://cs.stackexchange.com/questions/2016/how-to-convert-finite-automata-to-regular-expressions
// https://github.com/ferno/greenery/blob/bcc0a136335edbe94cd7725fc6e8cce0268d850c/fsm.py

/**
 * Converts FA to RE using Brzozowski's algebraic method using right regular equations.
 * Source : Melichar 2.122
 */
class Algebraic : public automaton::VisitableAutomatonBase::const_visitor_type
{
public:
	/**
	 * Performs conversion.
	 * @return regular expression equivalent to input automaton.
	 */
	static regexp::RegExp convert(const automaton::Automaton& automaton);

	template <class T>
	static regexp::UnboundedRegExp convert(const T& automaton);

private:
	void Visit(void*, const automaton::UnknownAutomaton&) const;
	void Visit(void*, const automaton::EpsilonNFA&) const;
	void Visit(void*, const automaton::NFA&) const;
	void Visit(void*, const automaton::DFA&) const;
	void Visit(void*, const automaton::ExtendedNFA&) const;
	void Visit(void*, const automaton::CompactNFA&) const;
	void Visit(void*, const automaton::InputDrivenNPDA&) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA&) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA&) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA&) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA&) const;
	void Visit(void*, const automaton::NPDA&) const;
	void Visit(void*, const automaton::SinglePopNPDA&) const;
	void Visit(void*, const automaton::DPDA&) const;
	void Visit(void*, const automaton::SinglePopDPDA&) const;
	void Visit(void*, const automaton::OneTapeDTM&) const;

	static const Algebraic ALGEBRAIC;
};

} /* namespace fa2re */

} /* namespace conversions */

#endif /* FA2RE_ALGEBRAIC_H_ */
