/*
 * BrzozowskiDerivation.h
 *
 *  Created on: 6. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef RE2RG_BRZOZOWSKIDERIVATION_H_
#define RE2RG_BRZOZOWSKIDERIVATION_H_

#include <grammar/Regular/RightRG.h>
#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

namespace conversions
{

namespace re2rg
{

/**
 * Converts reg. expression to right regular grammar using brzozowski derivation algorithm.
 * Source: Melichar 2.137
 */
class BrzozowskiDerivation : public regexp::VisitableRegExpBase::const_visitor_type
{
public:
	/**
	 * Performs conversion.
	 * @return right regular grammar equivalent to source regexp.
	 */
	static grammar::RightRG convert(const regexp::RegExp& regexp);

	template <class T>
	static grammar::RightRG convert(const T& regexp);

private:
	void Visit(void*, const regexp::FormalRegExp& regexp) const;
	void Visit(void*, const regexp::UnboundedRegExp& regexp) const;

	static const BrzozowskiDerivation BRZOZOWSKI_DERIVATION;
};

} /* namespace re2rg */

} /* namespace conversions */

#endif /* RE2RG_BRZOZOWSKIDERIVATION_H_ */
