/*
 * Glushkov.h
 *
 *  Created on: 11. 1. 2014
 *      Author: Tomas Pecka
 */

#ifndef RE2RG_GLUSHKOVNFA_H_
#define RE2RG_GLUSHKOVNFA_H_

#include <grammar/Grammar.h>
#include <grammar/Regular/RightRG.h>

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

namespace conversions
{

namespace re2rg
{

/**
 * Converts regular expression to right regular grammar using Glushkov algorithm.
 * Source: None yet.
 */
class GlushkovNFA : public regexp::VisitableRegExpBase::const_visitor_type
{
public:
    /**
     * Performs conversion.
     * @param regexp original regular expression
     * @return right regular grammar equivalent to source regexp.
     */
    static grammar::Grammar convert(const regexp::RegExp& regexp);

    static grammar::RightRG convert(const regexp::FormalRegExp& regexp);
    static grammar::RightRG convert(const regexp::UnboundedRegExp& regexp);

private:
    void Visit(void*, const regexp::FormalRegExp& regexp) const;
    void Visit(void*, const regexp::UnboundedRegExp& regexp) const;

    static const GlushkovNFA GLUSHKOV_NFA;
};

} /* namespace re2rg */

} /* namespace conversions */

#endif /* RE2RG_GLUSHKOVNFA_H_ */
