/*
 * RegExpEmpty.cpp
 *
 *  Created on: 19. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "RegExpEmpty.h"

namespace regexp
{

bool RegExpEmpty::languageIsEmpty(const regexp::RegExp& regexp)
{
	bool out;
	regexp.getData().Accept((void*) &out, RegExpEmpty::REG_EXP_EMPTY);
	return out;
}

bool RegExpEmpty::languageIsEmpty(const regexp::FormalRegExp& regexp)
{
	bool out;
	regexp.getRegExp().Accept((void*) &out, RegExpEmpty::REG_EXP_EMPTY);
	return out;
}

bool RegExpEmpty::languageIsEmpty(const regexp::UnboundedRegExp& regexp)
{
	bool out;
	regexp.getRegExp().Accept((void*) &out, RegExpEmpty::REG_EXP_EMPTY);
	return out;
}

// ----------------------------------------------------------------------------

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpAlternation& alternation) const
{
	bool &ret = *(bool*) data;

	for(const auto& element : alternation.getElements())
	{
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(data, *this);
		if(! ret)
		{
			ret = false;
			return;
		}
	}
	ret = true;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpConcatenation& concatenation) const
{
	bool &ret = *(bool*) data;

	for(const auto& element : concatenation.getElements())
	{
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(data, *this);
		if(ret)
		{
			ret = true;
			return;
		}
	}
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpIteration&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpSymbol&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpEmpty&) const
{
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpEpsilon&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

// ----------------------------------------------------------------------------

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpAlternation& alternation) const
{
	bool &ret = *(bool*) data;

	static_cast<const regexp::FormalRegExpElement&>(alternation.getLeftElement()).Accept(data, *this);
	bool retLeft = ret;

	static_cast<const regexp::FormalRegExpElement&>(alternation.getRightElement()).Accept(data, *this);
	bool retRight = ret;

	ret = retLeft && retRight;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpConcatenation& concatenation) const
{
	bool &ret = *(bool*) data;

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getLeftElement()).Accept(data, *this);
	bool retLeft = ret;

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getRightElement()).Accept(data, *this);
	bool retRight = ret;

	ret = retLeft || retRight;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpIteration&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpSymbol&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpEmpty&) const
{
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpEpsilon&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

// ----------------------------------------------------------------------------

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExp& regexp) const
{
	bool &ret = *(bool*) data;
	ret = RegExpEmpty::REG_EXP_EMPTY.languageIsEmpty(regexp);
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExp& regexp) const
{
	bool &ret = *(bool*) data;
	ret = RegExpEmpty::REG_EXP_EMPTY.languageIsEmpty(regexp);
}

const RegExpEmpty RegExpEmpty::REG_EXP_EMPTY;

} /* namespace regexp */
