/*
 * RegExpEmpty.h
 *
 *  Created on: 19. 1. 2014
 *	  Author: Tomas Pecka
 */

#ifndef REG_EXP_EMPTY_H_
#define REG_EXP_EMPTY_H_

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>

namespace regexp
{

/**
 * Determines whether regular expression is empty (regexp == \0)
 *
 */
class RegExpEmpty : public regexp::VisitableRegExpBase::const_visitor_type, regexp::FormalRegExpElement::const_visitor_type, regexp::UnboundedRegExpElement::const_visitor_type
{
public:
	static bool languageIsEmpty(const regexp::RegExp& regexp);

	static bool languageIsEmpty(const regexp::FormalRegExp& regexp);
	static bool languageIsEmpty(const regexp::UnboundedRegExp& regexp);

private:
	void Visit(void* data, const regexp::UnboundedRegExp& regexp) const;
	void Visit(void* data, const regexp::FormalRegExp& regexp) const;

	void Visit(void* data, const regexp::UnboundedRegExpAlternation& alternation) const;
	void Visit(void* data, const regexp::UnboundedRegExpConcatenation& concatenation) const;
	void Visit(void* data, const regexp::UnboundedRegExpIteration& iteration) const;
	void Visit(void* data, const regexp::UnboundedRegExpSymbol& symbol) const;
	void Visit(void* data, const regexp::UnboundedRegExpEmpty& empty) const;
	void Visit(void* data, const regexp::UnboundedRegExpEpsilon& epsilon) const;

	void Visit(void* data, const regexp::FormalRegExpAlternation& alternation) const;
	void Visit(void* data, const regexp::FormalRegExpConcatenation& concatenation) const;
	void Visit(void* data, const regexp::FormalRegExpIteration& iteration) const;
	void Visit(void* data, const regexp::FormalRegExpSymbol& symbol) const;
	void Visit(void* data, const regexp::FormalRegExpEmpty& empty) const;
	void Visit(void* data, const regexp::FormalRegExpEpsilon& epsilon) const;

	static const RegExpEmpty REG_EXP_EMPTY;
};

} /* namespace regexp */

#endif /* REG_EXP_EMPTY_H_ */
