/*
 * TrimCFG.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef TRIM_CFG_H_
#define TRIM_CFG_H_

#include <algorithm>
#include <deque>
#include <set>

#include <exception/AlibException.h>

#include <alphabet/Symbol.h>

namespace trim {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class TrimCFG {
public:
	/*
	 * Removes unreachable symbols - Melichar 3.9
	 */
	template<class T>
	static T removeUnreachableSymbols( const T & grammar );

	/**
	 * Removes unproductive (or useless - terminology) symbols - Melichar 3.12
	 */
	template<class T>
	static T removeUnproductiveSymbols( const T & grammar );

	/**
	 * Removes unproductive and useless symbols - Melichar 3.12
	 */
	template<class T>
	static T trim( const T & grammar );
};

}

#endif /* TRIM_CFG_H_ */
