#include <list>
#include "re2faTest.h"

#include "conversions/re2fa/BrzozowskiDerivation.h"
#include "conversions/re2fa/GlushkovNFA.h"
#include "conversions/re2fa/Thompson.h"
#include "conversions/fa2re/Algebraic.h"
#include "determinize/nfa/NFADeterminizer.h"
#include "minimize/dfa/MinimizeDFA.h"
#include "normalize/dfa/NormalizeDFA.h"
#include "epsilon/fsm/FSMEpsilonRemover.h"

#include "regexp/unbounded/UnboundedRegExp.h"
#include "regexp/RegExpFromStringParser.h"

#include "automaton/FSM/NFA.h"
#include <factory/DataFactory.hpp>

CPPUNIT_TEST_SUITE_REGISTRATION( re2faTest );

void re2faTest::setUp() {
}

void re2faTest::tearDown() {
}

void re2faTest::testThompson() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::RegExp regexp1( parser.parseValue() );

	automaton::Automaton enfa1 = conversions::re2fa::Thompson::convert(regexp1);

	regexp::RegExp regexp2( conversions::fa2re::Algebraic::convert(enfa1) );

	automaton::Automaton enfa2 = conversions::re2fa::Thompson::convert(regexp2);

	automaton::Automaton nfa1 = epsilon::FSMEpsilonRemover::remove(enfa1);
	automaton::Automaton nfa2 = epsilon::FSMEpsilonRemover::remove(enfa2);

	automaton::Automaton dfa1 = determinize::NFADeterminizer::determinize(nfa1);
	automaton::Automaton dfa2 = determinize::NFADeterminizer::determinize(nfa2);

	automaton::Automaton mdfa1 = minimize::MinimizeDFA::minimize(dfa1);
	automaton::Automaton mdfa2 = minimize::MinimizeDFA::minimize(dfa2);

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}

void re2faTest::testGlushkov() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::UnboundedRegExp regexp1( static_cast<const regexp::UnboundedRegExp &>( parser.parseValue().getData() ) );

    conversions::re2fa::GlushkovNFA glushkov1;
	automaton::NFA nfa1 = glushkov1.convert(regexp1);

	regexp::UnboundedRegExp regexp2( static_cast<const regexp::UnboundedRegExp &>( conversions::fa2re::Algebraic::convert(nfa1) ) );

	conversions::re2fa::GlushkovNFA glushkov2;
	automaton::NFA nfa2 = glushkov2.convert(regexp2);

	automaton::DFA dfa1 = determinize::NFADeterminizer::determinize(nfa1);
	automaton::DFA dfa2 = determinize::NFADeterminizer::determinize(nfa2);

	automaton::DFA mdfa1 = minimize::MinimizeDFA::minimize(dfa1);
	automaton::DFA mdfa2 = minimize::MinimizeDFA::minimize(dfa2);

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}

void re2faTest::testBrzozowski() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::RegExp regexp1( parser.parseValue() );

	automaton::Automaton nfa1 = conversions::re2fa::BrzozowskiDerivation::convert(regexp1);

	regexp::RegExp regexp2( conversions::fa2re::Algebraic::convert(static_cast<const automaton::NFA&>(nfa1.getData())) );

	automaton::Automaton nfa2 = conversions::re2fa::BrzozowskiDerivation::convert(regexp2);

	automaton::DFA mdfa1_1 = determinize::NFADeterminizer::determinize(static_cast<const automaton::NFA&>(nfa1.getData()));
	automaton::DFA mdfa1_2 = minimize::MinimizeDFA::minimize(mdfa1_1);
	automaton::DFA mdfa1_3 = normalize::NormalizeDFA::normalize(mdfa1_2);

	automaton::DFA mdfa2_1 = determinize::NFADeterminizer::determinize(static_cast<const automaton::NFA&>(nfa2.getData()));
	automaton::DFA mdfa2_2 = minimize::MinimizeDFA::minimize(mdfa2_1);
	automaton::DFA mdfa2_3 = normalize::NormalizeDFA::normalize(mdfa2_2);

	CPPUNIT_ASSERT( mdfa1_3 == mdfa2_3);
}
