#include <list>
#include "rg2rgTest.h"

#include "conversions/rg2rg/RightToLeftRegularGrammar.h"
#include "conversions/rg2rg/LeftToRightRegularGrammar.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_REGISTRATION( rg2rgTest );

void rg2rgTest::setUp() {
}

void rg2rgTest::tearDown() {
}

void rg2rgTest::testConversion() {
  grammar::RightRG rrGrammar(alphabet::symbolFrom(1));

  rrGrammar.addNonterminalSymbol(alphabet::symbolFrom(1));
  rrGrammar.addNonterminalSymbol(alphabet::symbolFrom(2));
  rrGrammar.addNonterminalSymbol(alphabet::symbolFrom(3));
  rrGrammar.addTerminalSymbol(alphabet::symbolFrom("a"));
  rrGrammar.addTerminalSymbol(alphabet::symbolFrom("b"));
  
  rrGrammar.addRule(alphabet::symbolFrom(1), std::make_pair(alphabet::symbolFrom("a"), alphabet::symbolFrom(2)));
  rrGrammar.addRule(alphabet::symbolFrom(2), std::make_pair(alphabet::symbolFrom("b"), alphabet::symbolFrom(3)));
  rrGrammar.addRule(alphabet::symbolFrom(3), alphabet::symbolFrom("a"));

  grammar::LeftRG lrGrammar = conversions::rg2rg::RightToLeftRegularGrammar::convert(rrGrammar);

  grammar::LeftRG lrGrammarRef(alphabet::symbolFrom(4));

  lrGrammarRef.addNonterminalSymbol(alphabet::symbolFrom(1));
  lrGrammarRef.addNonterminalSymbol(alphabet::symbolFrom(2));
  lrGrammarRef.addNonterminalSymbol(alphabet::symbolFrom(3));
  lrGrammarRef.addTerminalSymbol(alphabet::symbolFrom("a"));
  lrGrammarRef.addTerminalSymbol(alphabet::symbolFrom("b"));

  lrGrammarRef.addRule(alphabet::symbolFrom(2), std::make_pair(alphabet::symbolFrom(1), alphabet::symbolFrom("a")));
  lrGrammarRef.addRule(alphabet::symbolFrom(2), alphabet::symbolFrom("a"));
  lrGrammarRef.addRule(alphabet::symbolFrom(3), std::make_pair(alphabet::symbolFrom(2), alphabet::symbolFrom("b")));
  lrGrammarRef.addRule(alphabet::symbolFrom(4), std::make_pair(alphabet::symbolFrom(3), alphabet::symbolFrom("a")));

  CPPUNIT_ASSERT(lrGrammarRef == lrGrammar);
}

void rg2rgTest::testConversion2() {
  grammar::LeftRG lrGrammar(alphabet::symbolFrom(4));

  lrGrammar.addNonterminalSymbol(alphabet::symbolFrom(2));
  lrGrammar.addNonterminalSymbol(alphabet::symbolFrom(3));
  lrGrammar.addNonterminalSymbol(alphabet::symbolFrom(4));
  lrGrammar.addTerminalSymbol(alphabet::symbolFrom("a"));
  lrGrammar.addTerminalSymbol(alphabet::symbolFrom("b"));

  lrGrammar.addRule(alphabet::symbolFrom(2), alphabet::symbolFrom("a"));
  lrGrammar.addRule(alphabet::symbolFrom(3), std::make_pair(alphabet::symbolFrom(2), alphabet::symbolFrom("b")));
  lrGrammar.addRule(alphabet::symbolFrom(4), std::make_pair(alphabet::symbolFrom(3), alphabet::symbolFrom("a")));

  grammar::RightRG rrGrammar = conversions::rg2rg::LeftToRightRegularGrammar::convert(lrGrammar);

  grammar::RightRG rrGrammarRef(alphabet::symbolFrom(5));

  rrGrammarRef.addNonterminalSymbol(alphabet::symbolFrom(2));
  rrGrammarRef.addNonterminalSymbol(alphabet::symbolFrom(3));
  rrGrammarRef.addNonterminalSymbol(alphabet::symbolFrom(4));
  rrGrammarRef.addTerminalSymbol(alphabet::symbolFrom("a"));
  rrGrammarRef.addTerminalSymbol(alphabet::symbolFrom("b"));

  rrGrammarRef.addRule(alphabet::symbolFrom(5), std::make_pair(alphabet::symbolFrom("a"), alphabet::symbolFrom(2)));
  rrGrammarRef.addRule(alphabet::symbolFrom(2), std::make_pair(alphabet::symbolFrom("b"), alphabet::symbolFrom(3)));
  rrGrammarRef.addRule(alphabet::symbolFrom(3), alphabet::symbolFrom("a"));
  rrGrammarRef.addRule(alphabet::symbolFrom(3), std::make_pair(alphabet::symbolFrom("a"), alphabet::symbolFrom(4)));

  CPPUNIT_ASSERT(rrGrammarRef == rrGrammar);
}
