#include "RegExpEpsilonTest.h"

#include "regexp/RegExpEpsilon.h"
#include <sstream>
#include <regexp/RegExpFromStringParser.h>


CPPUNIT_TEST_SUITE_REGISTRATION( RegExpEpsilonTest );

void RegExpEpsilonTest::setUp() {
}

void RegExpEpsilonTest::tearDown() {
}

void RegExpEpsilonTest::testRegExpEpsilon() {
	{
		std::string input = "#E + ( (a #E) + a*)";
		std::stringstream inputs(input);
		regexp::RegExpFromStringParser parser(inputs);
		regexp::RegExp re = parser.parseValue();

		CPPUNIT_ASSERT(regexp::RegExpEpsilon::languageContainsEpsilon(re));
	}	
	{
		std::string input = "( a* )( a* )";
		std::stringstream inputs(input);
		regexp::RegExpFromStringParser parser(inputs);
		regexp::RegExp re = parser.parseValue();

		CPPUNIT_ASSERT(regexp::RegExpEpsilon::languageContainsEpsilon(re));
	}	
	{
		std::string input = "a + #0";
		std::stringstream inputs(input);
		regexp::RegExpFromStringParser parser(inputs);
		regexp::RegExp re = parser.parseValue();

		CPPUNIT_ASSERT(! regexp::RegExpEpsilon::languageContainsEpsilon(re));
	}	
	{
		std::string input = "#E + a #E + a*";
		std::stringstream inputs(input);
		regexp::RegExpFromStringParser parser(inputs);
		regexp::RegExp re = parser.parseValue();

		CPPUNIT_ASSERT(regexp::RegExpEpsilon::languageContainsEpsilon(re));
	}	
	{
		std::string input = "a* a*";
		std::stringstream inputs(input);
		regexp::RegExpFromStringParser parser(inputs);
		regexp::RegExp re = parser.parseValue();

		CPPUNIT_ASSERT(regexp::RegExpEpsilon::languageContainsEpsilon(re));
	}	
	{
		std::string input = "a s d #E + #E #0";
		std::stringstream inputs(input);
		regexp::RegExpFromStringParser parser(inputs);
		regexp::RegExp re = parser.parseValue();

		CPPUNIT_ASSERT(! regexp::RegExpEpsilon::languageContainsEpsilon(re));
	}	
	{
		std::string input = "a + #0";
		std::stringstream inputs(input);
		regexp::RegExpFromStringParser parser(inputs);
		regexp::RegExp re = parser.parseValue();

		CPPUNIT_ASSERT(! regexp::RegExpEpsilon::languageContainsEpsilon(re));
	}	
}
