/*
 * SymbolToXMLComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnciek
 */

#ifndef SYMBOL_TO_XML_COMPOSER_H_
#define SYMBOL_TO_XML_COMPOSER_H_

#include <list>
#include "Symbol.h"
#include "../sax/Token.h"
#include "../label/Label.h"

namespace alib {

template<typename T>
struct api;

} /* namespace alib */

namespace alphabet {

/**
 * This class contains methods to print XML representation of string to the output stream.
 */
class SymbolToXMLComposer {
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	std::list<sax::Token> compose(const SymbolBase& symbol) const;

	/**
	 * Prints XML representation of String to the output stream.
	 * @param string String to print
	 * @param out output stream to which print the String
	 */
	std::list<sax::Token> compose(const Symbol& symbol) const;

	std::list<sax::Token> compose(const LabeledSymbol& symbol) const;
	std::list<sax::Token> compose(const BlankSymbol& symbol) const;
	std::list<sax::Token> compose(const BottomOfTheStackSymbol& symbol) const;
	std::list<sax::Token> compose(const EndSymbol& symbol) const;

	template<typename T> friend class alib::api;
};

} /* namespace alphabet */

#endif /* SYMBOL_TO_XML_COMPOSER_H_ */
