/*
 * MultiInitialStates.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "MultiInitialStates.h"

#include <set>
#include <algorithm>
#include "../AutomatonException.h"

namespace automaton {

bool MultiInitialStates::addInitialState(const State& state) {
	if (states.find(state) == states.end()) {
		throw AutomatonException("State " + (std::string) state.getName() + " cannot be set as initial state. It is not present in the automaton.");
	}

	return initialStates.insert(state).second;
}

void MultiInitialStates::setInitialStates(const std::set<State>& newStates) {
	std::set<State> removed;
	std::set_difference(initialStates.begin(), initialStates.end(), newStates.begin(), newStates.end(), std::inserter(removed, removed.end()));

	std::set<State> added;
	std::set_difference(newStates.begin(), newStates.end(), initialStates.begin(), initialStates.end(), std::inserter(added, added.end()));
	
	for(const State& removedState : removed) {
		removeInitialState(removedState);
	}

	for(const State& addedState : added) {
		addInitialState(addedState);
	}
}

bool MultiInitialStates::removeInitialState(const State& state) {
	return initialStates.erase(state);
}

const std::set<State>& MultiInitialStates::getInitialStates() const {
	return initialStates;
}

} /* namespace automaton */

