/*
 * ContainerToXMLComposer.h
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#ifndef CONTAINER_TO_XML_COMPOSER_H_
#define CONTAINER_TO_XML_COMPOSER_H_

#include <list>
#include "../sax/Token.h"

#include "Container.h"
#include "ObjectsSet.h"
#include "ObjectsVector.h"
#include "ObjectsPair.h"

namespace alib {

template<typename T>
struct api;

} /* namespace alib */

namespace container {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class ContainerToXMLComposer {
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	std::list<sax::Token> compose(const ContainerBase& container) const;

	/**
	 * Prints XML representation of String to the output stream.
	 * @param string String to print
	 * @param out output stream to which print the String
	 */
	std::list<sax::Token> compose(const Container& container) const;

	template<typename T>
	std::list<sax::Token> compose(const std::set<T>& container) const;

	std::list<sax::Token> compose(const ObjectsSet& container) const;

	template<typename T>
	std::list<sax::Token> compose(const std::vector<T>& container) const;

	std::list<sax::Token> compose(const ObjectsVector& container) const;

	template<typename T, typename R>
	std::list<sax::Token> compose(const std::pair<T, R>& container) const;

	std::list<sax::Token> compose(const ObjectsPair& container) const;

	template<typename T, typename R>
	std::list<sax::Token> compose(const std::map<T, R>& container) const;

	std::list<sax::Token> compose(const ObjectsMap& container) const;

	template<typename T> friend class alib::api;
};

} /* namespace container */

#include "../Api.hpp"

namespace container {

template<typename T>
std::list<sax::Token> ContainerToXMLComposer::compose(const std::set<T>& container) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("set", sax::Token::TokenType::START_ELEMENT));

	for(const T& item : container) {
		out.splice(out.end(), alib::api<T>::compose(item));
	}

	out.push_back(sax::Token("set", sax::Token::TokenType::END_ELEMENT));
	return out;
}

template<typename T>
std::list<sax::Token> ContainerToXMLComposer::compose(const std::vector<T>& container) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("vector", sax::Token::TokenType::START_ELEMENT));

	for(const T& item : container) {
		out.splice(out.end(), alib::api<T>::compose(item));
	}

	out.push_back(sax::Token("vector", sax::Token::TokenType::END_ELEMENT));
	return out;
}

template<typename T, typename R>
std::list<sax::Token> ContainerToXMLComposer::compose(const std::pair<T, R>& container) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("pair", sax::Token::TokenType::START_ELEMENT));

	out.splice(out.end(), alib::api<T>::compose(container.first));
	out.splice(out.end(), alib::api<R>::compose(container.second));

	out.push_back(sax::Token("pair", sax::Token::TokenType::END_ELEMENT));
	return out;
}

template<typename T, typename R>
std::list<sax::Token> ContainerToXMLComposer::compose(const std::map<T, R>& container) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("map", sax::Token::TokenType::START_ELEMENT));

	for(const std::pair<T, R>& item : container) {
		out.splice(out.end(), alib::api<std::pair<T, R>>::compose(item));
	}

	out.push_back(sax::Token("map", sax::Token::TokenType::END_ELEMENT));
	return out;
}

} /* namespace container */

#endif /* CONTAINER_TO_XML_COMPOSER_H_ */

