#ifndef GRAMMAR_TO_STRING_COMPOSER_H_
#define GRAMMAR_TO_STRING_COMPOSER_H_

#include <iostream>
#include "Grammar.h"
#include "UnknownGrammar.h"

#include <map>
#include <list>
#include <set>

namespace grammar {

class GrammarToStringComposer : public VisitableGrammarBase::const_visitor_type {
	void Visit(void*, const UnknownGrammar& grammar) const;
	void Visit(void*, const LeftLG& grammar) const;
	void Visit(void*, const LeftRG& grammar) const;
	void Visit(void*, const RightLG& grammar) const;
	void Visit(void*, const RightRG& grammar) const;
	void Visit(void*, const LG& grammar) const;
	void Visit(void*, const CFG& grammar) const;
	void Visit(void*, const EpsilonFreeCFG& grammar) const;
	void Visit(void*, const CNF& grammar) const;
	void Visit(void*, const GNF& grammar) const;
	void Visit(void*, const CSG& grammar) const;
	void Visit(void*, const NonContractingGrammar& grammar) const;
	void Visit(void*, const ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const UnrestrictedGrammar& grammar) const;

public:
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	std::string compose(const GrammarBase& grammar) const;

	std::string compose(const Grammar& grammar) const;

	std::string compose(const UnknownGrammar& grammar) const;
	std::string compose(const LeftLG& grammar) const;
	std::string compose(const LeftRG& grammar) const;
	std::string compose(const RightLG& grammar) const;
	std::string compose(const RightRG& grammar) const;
	std::string compose(const LG& grammar) const;
	std::string compose(const CFG& grammar) const;
	std::string compose(const EpsilonFreeCFG& grammar) const;
	std::string compose(const CNF& grammar) const;
	std::string compose(const GNF& grammar) const;
	std::string compose(const CSG& grammar) const;
	std::string compose(const NonContractingGrammar& grammar) const;
	std::string compose(const ContextPreservingUnrestrictedGrammar& grammar) const;
	std::string compose(const UnrestrictedGrammar& grammar) const;
};

}

#endif /* GRAMMAR_TO_STRING_COMPOSER_H_ */
