/*
 * GrammarToXMLComposer.h
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#ifndef GRAMMAR_TO_XML_COMPOSER_H_
#define GRAMMAR_TO_XML_COMPOSER_H_

#include <string>
#include <list>
#include "Grammar.h"
#include "UnknownGrammar.h"
#include "Regular/LeftLG.h"
#include "Regular/LeftRG.h"
#include "Regular/RightLG.h"
#include "Regular/RightRG.h"
#include "ContextFree/LG.h"
#include "ContextFree/CFG.h"
#include "ContextFree/EpsilonFreeCFG.h"
#include "ContextFree/CNF.h"
#include "ContextFree/GNF.h"
#include "ContextSensitive/CSG.h"
#include "ContextSensitive/NonContractingGrammar.h"
#include "Unrestricted/ContextPreservingUnrestrictedGrammar.h"
#include "Unrestricted/UnrestrictedGrammar.h"
#include "../sax/Token.h"

namespace alib {

template<typename T>
struct api;

} /* namespace alib */

namespace grammar {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class GrammarToXMLComposer {
	void composeNonterminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const;
	void composeTerminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const;
	void composeInitialSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const;
	void composeGeneratesEpsilon(std::list<sax::Token>& out, bool generatesEpsilon) const;
	
	void composeRules(std::list<sax::Token>& out, const UnknownGrammar& grammar) const;
	void composeRules(std::list<sax::Token>& out, const UnrestrictedGrammar& grammar) const;
	void composeRules(std::list<sax::Token>& out, const ContextPreservingUnrestrictedGrammar& grammar) const;
	void composeRules(std::list<sax::Token>& out, const NonContractingGrammar& grammar) const;
	void composeRules(std::list<sax::Token>& out, const CSG& grammar) const;
	void composeRules(std::list<sax::Token>& out, const GNF& grammar) const;
	void composeRules(std::list<sax::Token>& out, const CNF& grammar) const;
	void composeRules(std::list<sax::Token>& out, const EpsilonFreeCFG& grammar) const;
	void composeRules(std::list<sax::Token>& out, const CFG& grammar) const;
	void composeRules(std::list<sax::Token>& out, const LG& grammar) const;
	void composeRules(std::list<sax::Token>& out, const LeftLG& grammar) const;
	void composeRules(std::list<sax::Token>& out, const LeftRG& grammar) const;
	void composeRules(std::list<sax::Token>& out, const RightLG& grammar) const;
	void composeRules(std::list<sax::Token>& out, const RightRG& grammar) const;

	void composeRuleLContext(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const;
	void composeRuleLHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const;
	void composeRuleRContext(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const;
	void composeRuleSingleSymbolLHS(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const;
	void composeRuleRHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const;
	void composeRuleOneOrTwoSymbolsRHS(std::list<sax::Token>& out, const std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>>& symbols) const;
	void composeRuleGNFRHS(std::list<sax::Token>& out, const std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>& symbols) const;
	void composeRuleLeftLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>>& symbols) const;
	void composeRuleRightLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>>& symbols) const;
	void composeRuleLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>& symbols) const;
	
	/**
	 * Prints XML representation of UnknownGrammar to the output stream.
	 * @param grammar grammar to print
	 * @return list of xml tokens representing the grammar
	 */
	std::list<sax::Token> compose(const GrammarBase& grammar) const;

	std::list<sax::Token> compose(const Grammar& grammar) const;

	std::list<sax::Token> compose(const UnknownGrammar& grammar) const;
	std::list<sax::Token> compose(const LeftLG& grammar) const;
	std::list<sax::Token> compose(const LeftRG& grammar) const;
	std::list<sax::Token> compose(const RightLG& grammar) const;
	std::list<sax::Token> compose(const RightRG& grammar) const;
	std::list<sax::Token> compose(const LG& grammar) const;
	std::list<sax::Token> compose(const CFG& grammar) const;
	std::list<sax::Token> compose(const EpsilonFreeCFG& grammar) const;
	std::list<sax::Token> compose(const CNF& grammar) const;
	std::list<sax::Token> compose(const GNF& grammar) const;
	std::list<sax::Token> compose(const CSG& grammar) const;
	std::list<sax::Token> compose(const NonContractingGrammar& grammar) const;
	std::list<sax::Token> compose(const ContextPreservingUnrestrictedGrammar& grammar) const;
	std::list<sax::Token> compose(const UnrestrictedGrammar& grammar) const;

	template<typename T> friend class alib::api;
};

} /* namespace grammar */

#endif /* GRAMMAR_TO_XML_COMPOSER_H_ */

