/*
 * UnknownRule.h
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#ifndef UNKNOWN_RULE_H_
#define UNKNOWN_RULE_H_

#include <vector>
#include "../alphabet/Symbol.h"

namespace grammar {

/**
 * Represents rewrite rule (production) of a Grammar.
 */
class UnknownRule {
private:
	std::vector<alphabet::Symbol> leftSide;
	std::vector<alphabet::Symbol> rightSide;
public:
	explicit UnknownRule();
	explicit UnknownRule(const std::vector<alphabet::Symbol>& leftSide, const std::vector<alphabet::Symbol>& rightSide);

	/**
	 * Sets left side of the rule.
	 * @param leftSide vector of symbols to set
	 */
	void setLeftHandSide(const std::vector<alphabet::Symbol>& leftSide);

	/**
	 * Sets right side of the rule.
	 * @param rightSide vector of symbols to set
	 */
	void setRightHandSide(const std::vector<alphabet::Symbol>& rightSide);

	/**
	 * @return vector of symbols on the left side of the UnknownRule
	 */
	const std::vector<alphabet::Symbol>& getLeftHandSide() const;

	/**
	 * @return vector of symbols on the right side of the UnknownRule
	 */
	const std::vector<alphabet::Symbol>& getRightHandSide() const;

	/**
	 * Checks that UnknownRule contains given symbol.
	 * @param symbol Symbol to find
	 * @return true when UnknownRule contains the symbol, false otherwise
	 */
	bool containsSymbol(const alphabet::Symbol& symbol) const;

	/**
	 * Converts rule into human readable string.
	 * @return string in format [ leftSide -> rightSide ]
	 */
	std::string toString() const;

	bool operator <(const UnknownRule& other) const;
	bool operator ==(const UnknownRule& other) const;
	bool operator !=(const UnknownRule& other) const;

	friend std::ostream& operator<<(std::ostream&, const UnknownRule&);
};

} /* namespace grammar */

#endif /* UNKNOWN_RULE_H_ */
