/*
 * TerminalNonterminalAlphabetAlphabetInitialSymbol.cpp
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#include "TerminalNonterminalAlphabetInitialSymbol.h"
#include "../GrammarException.h"
#include "../../alphabet/LabeledSymbol.h"
#include "../../label/Label.h"
#include "../../alphabet/LabeledSymbol.h"
#include "../../alphabet/Symbol.h"
#include <climits>
#include <algorithm>

namespace grammar {

TerminalNonterminalAlphabetInitialSymbol::TerminalNonterminalAlphabetInitialSymbol(const alphabet::Symbol& initialSymbol) : initialSymbol(initialSymbol) {
	addNonterminalSymbol(initialSymbol);
}

bool TerminalNonterminalAlphabetInitialSymbol::addTerminalSymbol(const alphabet::Symbol& symbol) {
	if(nonterminalAlphabet.find(symbol) != nonterminalAlphabet.end()){
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" is nonterminal symbol.");
	}

	return terminalAlphabet.insert(symbol).second;
}

const std::set<alphabet::Symbol>& TerminalNonterminalAlphabetInitialSymbol::getTerminalAlphabet() const {
	return terminalAlphabet;
}

void TerminalNonterminalAlphabetInitialSymbol::setTerminalAlphabet(const std::set<alphabet::Symbol>& alphabet) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(terminalAlphabet.begin(), terminalAlphabet.end(), alphabet.begin(), alphabet.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(alphabet.begin(), alphabet.end(), terminalAlphabet.begin(), terminalAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeTerminalSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addTerminalSymbol(addedSymbol);
	}
}

bool TerminalNonterminalAlphabetInitialSymbol::addNonterminalSymbol(const alphabet::Symbol& symbol) {
	if(terminalAlphabet.find(symbol) != terminalAlphabet.end()){
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" is terminal symbol.");
	}

	return nonterminalAlphabet.insert(symbol).second;
}

const std::set<alphabet::Symbol>& TerminalNonterminalAlphabetInitialSymbol::getNonterminalAlphabet() const {
	return nonterminalAlphabet;
}

void TerminalNonterminalAlphabetInitialSymbol::setNonterminalAlphabet(const std::set<alphabet::Symbol>& alphabet) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(nonterminalAlphabet.begin(), nonterminalAlphabet.end(), alphabet.begin(), alphabet.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(alphabet.begin(), alphabet.end(), nonterminalAlphabet.begin(), nonterminalAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeNonterminalSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addNonterminalSymbol(addedSymbol);
	}
}

const alphabet::Symbol& TerminalNonterminalAlphabetInitialSymbol::getInitialSymbol() const{
	return initialSymbol;
}

void TerminalNonterminalAlphabetInitialSymbol::setInitialSymbol(const alphabet::Symbol& symbol) {
	if(nonterminalAlphabet.find(symbol) == nonterminalAlphabet.end()) {
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" isn't nonterminal symbol.");
	}

	initialSymbol = symbol;
}

} /* namespace grammar */
