/*
 * LabelPairLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "LabelPairLabel.h"
#include "../std/pair.hpp"
#include <sstream>

namespace label {

LabelPairLabel::LabelPairLabel(const std::pair<Label, Label>& label) : label(std::move(label)) {

}

LabelPairLabel::LabelPairLabel(std::pair<Label, Label>&& label) : label(std::move(label)) {

}

LabelBase* LabelPairLabel::clone() const {
	return new LabelPairLabel(*this);
}

LabelBase* LabelPairLabel::plunder() && {
	return new LabelPairLabel(std::move(*this));
}

const std::pair<Label, Label>& LabelPairLabel::getData() const {
	return label;
}

bool LabelPairLabel::operator<(const alib::ObjectBase& other) const {
	return other > *this;
}

bool LabelPairLabel::operator==(const alib::ObjectBase& other) const {
	return other == *this;
}

bool LabelPairLabel::operator >(const alib::ObjectBase& other) const {
	return other < *this;
}


bool LabelPairLabel::operator <(const LabelPairLabel& other) const {
	return label < other.label;
}

bool LabelPairLabel::operator ==(const LabelPairLabel& other) const {
	return label == other.label;
}

void LabelPairLabel::operator>>(std::ostream& out) const {
	out << "(LabelPairLabel " << label << ")";
}

LabelPairLabel::operator std::string() const {
	std::stringstream ss;
	ss << "<";
	ss << (std::string) label.first;
	ss << ",";
	ss << (std::string) label.second;
	ss << "}";
	return std::move(ss).str();
}

} /* namespace label */

