/*
 * LabelToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelToStringComposer.h"
#include "../primitive/PrimitiveToStringComposer.h"
#include <algorithm>
#include "PrimitiveLabel.h"
#include "HexavigesimalLabel.h"
#include "ObjectLabel.h"
#include "LabelSetLabel.h"
#include "LabelPairLabel.h"

#include "../exception/AlibException.h"

namespace label {

void LabelToStringComposer::Visit(void* userData, const PrimitiveLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	primitive::PrimitiveToStringComposer composer;
	out << composer.compose(label.getData());
}

void LabelToStringComposer::Visit(void* userData, const HexavigesimalLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << (std::string) label;
}

void LabelToStringComposer::Visit(void* userData, const ObjectLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);
	out << (std::string) label;
}

void LabelToStringComposer::Visit(void* userData, const LabelSetLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << '[';
	bool first = true;
	for(const Label& innerLabel : label.getData()) {
		if(!first)
			out << ", ";
		else
			first = false;
		innerLabel.getData().Accept(userData, *this);
	}
	out << ']';
}

void LabelToStringComposer::Visit(void* userData, const LabelPairLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << '<';
	label.getData().first.getData().Accept(userData, *this);
	out << ", ";
	label.getData().second.getData().Accept(userData, *this);
	out << '>';
}

std::string LabelToStringComposer::compose(const Label& label) {
	std::stringstream out;
	label.getData().Accept((void*) &out, *this);
	return std::move(out).str();
}

} /* namespace label */
