/*
 * LabelToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelToXMLComposer.h"
#include "../std/itos.h"

#include "../Api.hpp"

namespace label {

std::list<sax::Token> LabelToXMLComposer::compose(const Label& label) const {
	std::list<sax::Token> out;
	label.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const LabelBase& label) const {
	std::list<sax::Token> out;
	label.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const PrimitiveLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("PrimitiveLabel", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<primitive::Primitive>::compose(label.getData()));
	out.push_back(sax::Token("PrimitiveLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const HexavigesimalLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("HexavigesimalLabel", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::itos(label.getData()), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token("HexavigesimalLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const ObjectLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("ObjectLabel", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alib::Object>::compose(label.getData()));
	out.push_back(sax::Token("ObjectLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const LabelSetLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("LabelSetLabel", sax::Token::TokenType::START_ELEMENT));
	for(const Label& innerLabel : label.getData()) {
		out.splice(out.end(), alib::api<label::Label>::compose(innerLabel));
	}
	out.push_back(sax::Token("LabelSetLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const LabelPairLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("LabelPairLabel", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<label::Label>::compose(label.getData().first));
	out.splice(out.end(), alib::api<label::Label>::compose(label.getData().second));
	out.push_back(sax::Token("LabelPairLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

} /* namespace label */
