/*
 * NextLabel.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "NextLabel.h"
#include "../primitive/NextPrimitive.h"
#include "PrimitiveLabel.h"
#include "HexavigesimalLabel.h"
#include "../std/itos.h"
#include "../exception/AlibException.h"

namespace label {

void NextLabel::Visit(void* userData, const PrimitiveLabel& label) {
	Label* &out = *((Label**) userData);

	primitive::NextPrimitive next;
	out = new Label(PrimitiveLabel(next.nextPrimitive(label.getData())));
}

void NextLabel::Visit(void* userData, const HexavigesimalLabel& label) {
	Label* &out = *((Label**) userData);

	out = new Label(HexavigesimalLabel(label.getData() + 1));
}

void NextLabel::Visit(void*, const ObjectLabel&) {
	throw exception::AlibException("LabelSet has no next Label.");
}

void NextLabel::Visit(void*, const LabelSetLabel&) {
	throw exception::AlibException("LabelSet has no next Label.");
}

void NextLabel::Visit(void*, const LabelPairLabel&) {
	throw exception::AlibException("LabelPair has no next Label.");
}

Label NextLabel::nextLabel(const Label& label) {
	Label* out;
	label.getData().Accept((void*) &out, *this);

	Label nextLabel(std::move(*out));
	delete out;
	return std::move(nextLabel);
}

} /* namespace label */
