/*
 * ObjectLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef OBJECT_LABEL_H_
#define OBJECT_LABEL_H_

#include <ostream>

#include "LabelBase.h"
#include "../object/Object.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class ObjectLabel : public std::acceptor<ObjectLabel, VisitableLabelBase, std::acceptor<ObjectLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	alib::Object label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit ObjectLabel(const alib::Object& label);

	explicit ObjectLabel(alib::Object&& label);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	virtual bool operator<(const alib::ObjectBase& other) const;
	
	virtual bool operator>(const alib::ObjectBase& other) const;

	virtual bool operator==(const alib::ObjectBase& other) const;

	/**
	 * @return name of the symbol
	 */
	const alib::Object& getData() const;

	virtual bool operator <(const ObjectLabel& other) const;
	
	virtual bool operator ==(const ObjectLabel& other) const;
	
	virtual void operator>>(std::ostream&) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace label */

#endif /* OBJECT_LABEL_H_ */
