/*
 * ObjectBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef OBJECT_BASE_H_
#define OBJECT_BASE_H_

#include "../std/visitor.hpp"
#include "../common/base.hpp"

namespace alib {

class Void;

}

namespace exception {

class AlibException;

}

namespace automaton {

class UnknownAutomaton;
class DFA;
class NFA;
class EpsilonNFA;
class CompactNFA;
class ExtendedNFA;
class DPDA;
class SinglePopDPDA;
class NPDA;
class SinglePopNPDA;
class InputDrivenNPDA;
class VisiblyPushdownDPDA;
class VisiblyPushdownNPDA;
class RealTimeHeightDeterministicDPDA;
class RealTimeHeightDeterministicNPDA;
class OneTapeDTM;

}

namespace grammar {

class UnknownGrammar;
class LeftLG;
class LeftRG;
class RightLG;
class RightRG;
class LG;
class CFG;
class EpsilonFreeCFG;
class CNF;
class GNF;
class CSG;
class NonContractingGrammar;
class ContextPreservingUnrestrictedGrammar;
class UnrestrictedGrammar;

}

namespace label {

class PrimitiveLabel;
class ObjectLabel;
class HexavigesimalLabel;
class LabelSetLabel;
class LabelPairLabel;

}

namespace regexp {

class UnboundedRegExp;
class FormalRegExp;

}

namespace string {

class LinearString;
class CyclicString;
class Epsilon;

}

namespace alphabet {

class LabeledSymbol;
class BlankSymbol;
class BottomOfTheStackSymbol;
class EndSymbol;

}

namespace container {

class ObjectsSet;
class ObjectsVector;
class ObjectsPair;
class ObjectsMap;

}

namespace primitive {

class String;
class Integer;
class Character;

}

namespace alib {

typedef std::acceptor_base<
			Void,
			exception::AlibException,
			automaton::UnknownAutomaton, automaton::DFA, automaton::NFA, automaton::EpsilonNFA, automaton::CompactNFA, automaton::ExtendedNFA, automaton::DPDA, automaton::SinglePopDPDA, automaton::InputDrivenNPDA, automaton::VisiblyPushdownDPDA, automaton::VisiblyPushdownNPDA, automaton::RealTimeHeightDeterministicDPDA, automaton::RealTimeHeightDeterministicNPDA, automaton::NPDA, automaton::SinglePopNPDA, automaton::OneTapeDTM,
			grammar::UnknownGrammar, grammar::LeftLG, grammar::LeftRG, grammar::RightLG, grammar::RightRG, grammar::LG, grammar::CFG, grammar::EpsilonFreeCFG, grammar::CNF, grammar::GNF, grammar::CSG, grammar::NonContractingGrammar, grammar::ContextPreservingUnrestrictedGrammar, grammar::UnrestrictedGrammar,
			label::PrimitiveLabel, label::HexavigesimalLabel, label::ObjectLabel, label::LabelSetLabel, label::LabelPairLabel,
			regexp::UnboundedRegExp, regexp::FormalRegExp,
			string::Epsilon, string::LinearString, string::CyclicString,
			alphabet::LabeledSymbol, alphabet::BlankSymbol, alphabet::BottomOfTheStackSymbol, alphabet::EndSymbol,
			container::ObjectsSet, container::ObjectsVector, container::ObjectsPair, container::ObjectsMap,
			primitive::String, primitive::Integer, primitive::Character
	> VisitableObjectBase;

class ObjectBase :
	public alib::base<
			ObjectBase,
			Void,
			exception::AlibException,
			automaton::UnknownAutomaton, automaton::DFA, automaton::NFA, automaton::EpsilonNFA, automaton::CompactNFA, automaton::ExtendedNFA, automaton::DPDA, automaton::SinglePopDPDA, automaton::InputDrivenNPDA, automaton::VisiblyPushdownDPDA, automaton::VisiblyPushdownNPDA, automaton::RealTimeHeightDeterministicDPDA, automaton::RealTimeHeightDeterministicNPDA, automaton::NPDA, automaton::SinglePopNPDA, automaton::OneTapeDTM,
			grammar::UnknownGrammar, grammar::LeftLG, grammar::LeftRG, grammar::RightLG, grammar::RightRG, grammar::LG, grammar::CFG, grammar::EpsilonFreeCFG, grammar::CNF, grammar::GNF, grammar::CSG, grammar::NonContractingGrammar, grammar::ContextPreservingUnrestrictedGrammar, grammar::UnrestrictedGrammar,
			label::PrimitiveLabel, label::HexavigesimalLabel, label::ObjectLabel, label::LabelSetLabel, label::LabelPairLabel,
			regexp::UnboundedRegExp, regexp::FormalRegExp,
			string::Epsilon, string::LinearString, string::CyclicString,
			alphabet::LabeledSymbol, alphabet::BlankSymbol, alphabet::BottomOfTheStackSymbol, alphabet::EndSymbol,
			container::ObjectsSet, container::ObjectsVector, container::ObjectsPair, container::ObjectsMap,
			primitive::String, primitive::Integer, primitive::Character
	>, public VisitableObjectBase {
};

} /* namespace alib */

#endif /* OBJECT_BASE_H_ */
