/*
 * ObjectFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "ObjectFromXMLParser.h"

#include "../sax/ParserException.h"

#include "../Api.hpp"
#include <iostream>

namespace alib {

Object ObjectFromXMLParser::parseObject(std::list<sax::Token>& input) const {
	if(alib::FromXMLParsers::symbolParser.first(input)) {
		alphabet::Symbol symbol = alib::api<alphabet::Symbol>::parse(input);
		Object res(std::move(symbol.getData()));
		return res;
	} else if(alib::FromXMLParsers::exceptionParser.first(input)) {
		exception::AlibException exception = alib::api<exception::AlibException>::parse(input);
		Object res(std::move(exception));
		return res;
	} else if(alib::FromXMLParsers::labelParser.first(input)) {
		label::Label label = alib::api<label::Label>::parse(input);
		Object res(std::move(label.getData()));
		return res;
	} else if(alib::FromXMLParsers::regexpParser.first(input)) {
		regexp::RegExp regexp = alib::api<regexp::RegExp>::parse(input);
		Object res(std::move(regexp.getData()));
		return res;
	} else if(alib::FromXMLParsers::stringParser.first(input)) {
		string::String string = alib::api<string::String>::parse(input);
		Object res(std::move(string.getData()));
		return res;
	} else if(alib::FromXMLParsers::automatonParser.first(input)) {
		automaton::Automaton automaton = alib::api<automaton::Automaton>::parse(input);
		Object res(std::move(automaton.getData()));
		return res;
	} else if(alib::FromXMLParsers::grammarParser.first(input)) {
		grammar::Grammar grammar = alib::api<grammar::Grammar>::parse(input);
		Object res(std::move(grammar.getData()));
		return res;
	} else if(alib::FromXMLParsers::containerParser.first(input)) {
		container::Container container = alib::api<container::Container>::parse(input);
		Object res(std::move(container.getData()));
		return res;
	} else if(alib::FromXMLParsers::primitiveParser.first(input)) {
		primitive::Primitive primitive = alib::api<primitive::Primitive>::parse(input);
		Object res(std::move(primitive.getData()));
		return res;
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "Void")) {
		Object res(std::move(parseVoid(input)));
		return res;
	} else {
		throw exception::AlibException("Unknown element in xml");
	}
}

Void ObjectFromXMLParser::parseVoid(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "Void");
	popToken(input, sax::Token::TokenType::END_ELEMENT, "Void");

	return Void {};
}

bool ObjectFromXMLParser::first(std::list<sax::Token>& input) {
	return alib::FromXMLParsers::symbolParser.first(input) || alib::FromXMLParsers::exceptionParser.first(input) || alib::FromXMLParsers::labelParser.first(input) || alib::FromXMLParsers::regexpParser.first(input) || alib::FromXMLParsers::stringParser.first(input) || alib::FromXMLParsers::automatonParser.first(input) || alib::FromXMLParsers::grammarParser.first(input) || alib::FromXMLParsers::containerParser.first(input) || isToken(input, sax::Token::TokenType::START_ELEMENT, "Void");
}

} /* namespace alib */
