/*
 * Character.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_CHARACTER_H_
#define PRIMITIVE_CHARACTER_H_

#include <string>
#include <ostream>

#include "PrimitiveBase.h"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Character : public std::acceptor<Character, VisitablePrimitiveBase, std::acceptor<Character, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	char data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Character(char data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	virtual bool operator<(const alib::ObjectBase& other) const;

	virtual bool operator>(const alib::ObjectBase& other) const;

	virtual bool operator==(const alib::ObjectBase& other) const;

	/**
	 * @return name of the symbol
	 */
	char getData() const;

	virtual bool operator <(const Character& other) const;

	virtual bool operator ==(const Character& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace primitive */

#endif /* PRIMITIVE_CHARACTER_H_ */

