/*
 * PrimitiveFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveFromStringParser.h"
#include "../exception/AlibException.h"
#include "String.h"
#include "Integer.h"
#include "Character.h"
#include <algorithm>

namespace primitive {

PrimitiveFromStringParser::PrimitiveFromStringParser(std::istream& input) : m_Lexer(input) {

}

Primitive PrimitiveFromStringParser::parse() {
	return parse(std::set<FEATURES>({FEATURES::STRING, FEATURES::CHAR, FEATURES::INTEGER}));
}

Primitive PrimitiveFromStringParser::parse(const std::set<FEATURES>& features) {
	PrimitiveFromStringLexer::Token token = m_Lexer.token();
	switch(token.type) {
	case PrimitiveFromStringLexer::TokenType::TEOF:
		throw exception::AlibException("Unexpected end of file");
	case PrimitiveFromStringLexer::TokenType::STRING:
		if(!features.count(FEATURES::STRING)) throw exception::AlibException();
		return Primitive(String(token.value));
	case PrimitiveFromStringLexer::TokenType::CHAR:
		if(!features.count(FEATURES::CHAR)) throw exception::AlibException();
		return Primitive(Character(token.value[0]));
	case PrimitiveFromStringLexer::TokenType::INTEGER:
		if(!features.count(FEATURES::INTEGER)) throw exception::AlibException();
		return Primitive(Integer(std::stoi(token.value)));
	case PrimitiveFromStringLexer::TokenType::ERROR:
		break;
	}
	throw exception::AlibException("Unrecognised input \"" + token.value + "\"");
}

bool PrimitiveFromStringParser::next() {
	PrimitiveFromStringLexer::Token token = m_Lexer.next().token();
	if(token.type == PrimitiveFromStringLexer::TokenType::STRING || token.type == PrimitiveFromStringLexer::TokenType::CHAR || token.type == PrimitiveFromStringLexer::TokenType::INTEGER) {
		return true;
	} else {
		return false;
	}
}

bool PrimitiveFromStringParser::first() {
	PrimitiveFromStringLexer::Token token = m_Lexer.next().token();
	if(token.type == PrimitiveFromStringLexer::TokenType::STRING || token.type == PrimitiveFromStringLexer::TokenType::CHAR || token.type == PrimitiveFromStringLexer::TokenType::INTEGER) {
		return true;
	} else {
		return false;
	}
}

bool PrimitiveFromStringParser::last() {
	PrimitiveFromStringLexer::Token token = m_Lexer.next().token();
	if(token.type == PrimitiveFromStringLexer::TokenType::TEOF) {
		return true;
	} else {
		return false;
	}
}

} /* namespace primitive */
