/*
 * PrimitiveFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveFromXMLParser.h"
#include "../sax/ParserException.h"
#include <algorithm>

namespace primitive {

Primitive PrimitiveFromXMLParser::parsePrimitive(std::list<sax::Token>& input) const {
	return parsePrimitive(input, std::set<FEATURES>({FEATURES::STRING, FEATURES::CHAR, FEATURES::INTEGER}));
}

Primitive PrimitiveFromXMLParser::parsePrimitive(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "Integer")) {
		if(!features.count(FEATURES::INTEGER)) throw exception::AlibException();
		return Primitive(parseInteger(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "String")) {
		if(!features.count(FEATURES::STRING)) throw exception::AlibException();
		return Primitive(parseString(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "Character")) {
		if(!features.count(FEATURES::CHAR)) throw exception::AlibException();
		return Primitive(parseCharacter(input));
	} else {
		throw sax::ParserException(sax::Token("Integer, String, Character", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

bool PrimitiveFromXMLParser::first(std::list<sax::Token>& input) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "Integer") || isToken(input, sax::Token::TokenType::START_ELEMENT, "String") || isToken(input, sax::Token::TokenType::START_ELEMENT, "Character")) {
		return true;
	} else {
		return false;
	}
}

Integer PrimitiveFromXMLParser::parseInteger(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "Integer");
	Integer data(std::stoi(popTokenData(input, sax::Token::TokenType::CHARACTER)));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "Integer");
	return data;
}

Character PrimitiveFromXMLParser::parseCharacter(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "Character");
	Character data(popTokenData(input, sax::Token::TokenType::CHARACTER)[0]);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "Character");
	return data;
}

String PrimitiveFromXMLParser::parseString(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "String");
	String data(popTokenData(input, sax::Token::TokenType::CHARACTER));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "String");
	return data;
}

} /* namespace primitive */
