/*
 * StringLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_STRING_H_
#define PRIMITIVE_STRING_H_

#include <string>
#include <ostream>

#include "PrimitiveBase.h"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class String : public std::acceptor<String, VisitablePrimitiveBase, std::acceptor<String, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	std::string data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit String(const std::string& data);

	explicit String(std::string&& data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	virtual bool operator<(const alib::ObjectBase& other) const;

	virtual bool operator>(const alib::ObjectBase& other) const;

	virtual bool operator==(const alib::ObjectBase& other) const;

	/**
	 * @return name of the symbol
	 */
	const std::string& getData() const;

	virtual bool operator <(const String& other) const;

	virtual bool operator ==(const String& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace primitive */

#endif /* PRIMITIVE_STRING_H_ */

