/*
 * Alphabet.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_ALPHABET_H_
#define STRING_ALPHABET_H_

#include <set>
#include "../../alphabet/Symbol.h"


namespace string {

/**
 * Abstract base class for all strings. Contains common elements of strings.
 */
class StringAlphabet {
protected:
	std::set<alphabet::Symbol> alphabet;
public:
	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 */
	bool addSymbolToAlphabet(const alphabet::Symbol& symbol);

	/**
	 * Adds input symbols to input alphabet.
	 * @param symbols Symbol to add
	 */
	void addSymbolsToAlphabet(const std::set<alphabet::Symbol>& symbols);

	/**
	 * Sets input symbols of the string.
	 * @param symbols Symbols to set
	 */
	void setAlphabet(const std::set<alphabet::Symbol>& symbols);

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 * or when symbol is part of the transition
	 */
	virtual bool removeSymbolFromAlphabet(const alphabet::Symbol& symbol) = 0;

	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::Symbol>& getAlphabet() const;

};

} /* namespace string */

#endif /* STRING_ALPHABET_H_ */

